;################################################################# SENDBIN.ASL
; Purpose.....: Send data in binary format over USART on EOM-MPF.
;		Serial parameters are 4800,N,8,1.
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line: ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:31.08.2024
; Created
;***************************************************************** end history

;*********************************************************************** notes
;******************************************************************* end notes

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file
		
;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING	ON
		
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
wSBCS:		EQU	0F000H		; code (B340-B48F if IOM-ROM)
wSBDS:		EQU	0FD00H		; data on top of MPF RAM

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
wSBVS:		EQU	wSBDS		; variables start address
vSBFLG:		EQU	wSBVS		; various flags
iSBFLI:		EQU	0		;  bit0: Address input mode
vSBFA:		EQU	vSBFLG + 1	; from address for success display
vSBDB:		EQU	vSBFA  + 2	; display buffer for address input
wSBVE:		EQU	vSBDB  + dbMCPL	; variables end address

;======================================== check overlap with library variables
		IF	wSBVE > dwLVST && MomPass = 2
		WARNING	"Variables overlap with library:\{wSBVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wSBCS

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== SB()
; Purpose.....: Main program
; Labels......: S)end B)inary
;=============================================================================
SB:		CALL	dfLLS		; initialize library variables

		LD	HL,0
		LD	(vSBFA),HL	; clear start address
		LD	IY,vSBFLG	; IY= pointer to flags
		LD	(IY),0		; clear flags

		LD	IX,tSBSO	; sign-on message
		LD	B,030H		; 30H ~1s
		CALL	dfLDT		; show it

;----------------------------------------------------------------- setup USART
		LD	B,dbUB4K	; baudrate 4800
		LD	C,dbU8N1	; 8 data, no parity, 1 stop	
		CALL	dfLUS		; setup USART

;--------------------------------------- get start/end addresses from keyboard
SBADR:		LD	HL,wSBTCPS	; HL= text addresses
		LD	DE,vSBDB	; DE= display buffer address
		LD	BC,bSBTCPZ	; BC= text length
		LDIR			; copy text to RAM buffer

SBADRFR:	LD	IX,vSBDB	; IX= text buffer address
		LD	C,bSBADRFP	; cursor position from address
		RES	iSBFLI,(IY)	; from-input active
		JR	SBADRGET	; read it

SBADRTO:	LD	IX,vSBDB	; IX= text buffer address= reinit
		LD	C,bSBADRTP	; C= cursor position to address
		SET	iSBFLI,(IY)	; to-input active

SBADRGET:	LD	A,2		; set hex mode
		LD	B,4		; use four digits
		SCF			; buffer clear
SBADRGETL:	CALL	dfLDKR		; call editor
		LD	IX,0		; set continue mode
		JR	C,SBADRQ	; aborted with SHIFT&CONTROL
		JR	NZ,SBADRGETL	; not CR, continue reading

        	BIT	iSBFLI,(IY)	; from address input?
		JR	NZ,SBADRCR	; no, to
		EX	DE,HL		; yes, HL= from address
		LD	(vSBFA),HL	; store for end display
		JR	SBADRTO		; now get to-address

SBADRCR:	CALL	dfLHHLDE	; compare HL and DE
		JR	Z,SBADRTO	; same is not allowed
		JR	NC,SBADRTO	; DE < HL, error, reread
		JR	SBSEND
					
SBADRQ:		BIT	iSBFLI,(IY)	; inputting to-address?
		JR	NZ,SBADR	; yes, start over
		JR	SBQ		; no, quit

;---------------------------------------------------------- start sending data
SBSEND:		LD	A,'-'
		CALL	SBDP		; display sending pattern

SBSENDL:	CALL	dfLKSAC		; check for abort request
		JR	C,SBQ		; shift and control pressed, exit

		LD	A,(HL)		; get next byte to send
		CALL	dfLUTB		; try to send the byte
		JR	C,SBSENDL	; not sent, retry
		CALL	dfLHHLDE	; sent, compare HL and DE
		JR	Z,SBDONE	; same, done
		INC	HL		; bump address pointer
		JR	SBSENDL		; send next byte

;-------------------------------------- success, display from and to addresses
SBDONE:		PUSH	HL		; save last read address
		CALL	dfMCLR		; clear display buffer
		LD	HL,tSBFP	; from prompt
		CALL	dfMMSG		; to display buffer
		LD	HL,(vSBFA)	; HL= from address
		CALL	dfMB4H		; to display buffer
		LD	HL,tSBTP	; to prompt
		CALL	dfMMSG		; to display buffer
		POP	HL		; recall last read address
		CALL	dfMB4H		; to display buffer

;---------------------------------- display exit message and return to monitor
SBX:		LD	IX,dvMDB	; IX= display buffer address
		CALL	dfMSC		; print message and wait for key

SBQ:  		JP	dwMRET		; monitor warm start

;-------------------------------------------------------------- text constants
tSBSO:		DB	'    SB V24-09-01    '
tSBFP:		DB	'FROM ',                0DH
tSBTP:		DB	'  TO ',                0DH

;------------------------------------------ text constants to be copied to RAM
wSBTCPS:	EQU	$
tSBAG:		DB	'FROM       TO       ', 0DH
bSBADRFP:	EQU	5	
bSBADRTP:	EQU	14
wSBTCPE:	EQU	$
bSBTCPZ:	EQU	wSBTCPE-wSBTCPS
;==================================================================== eoM RB()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ subroutines
;====================================================================== SBDP()
; Purpose.....:	Display static pattern in the center of the display to
;		signal progress.
; Input.......: A: Pattern character to display
; Registers...:	AF destroyed
; Labels......: R)ead B)in D)isplay P)attern
;=============================================================================
SBDP:		PUSH	DE
		PUSH	BC

		LD	D,00011111B	; digits 8-16
		LD	E,10000000B	; digits 1-8
		LD	C,00000000B	; digits 17-20
		LD	B,A		; save pattern
		CALL	dfLDP		; display the pattern

		POP	BC
		POP	DE
		RET
;================================================================== eoF RBDP()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end subroutines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ size info
wSBCSE:		EQU	$
		IF	MomPass = 2
		MESSAGE "Code from \{wSBCS} to \{wSBCSE} Size \{wSBCSE-wSBCS}"
		MESSAGE "Data from \{wSBDS} to \{wSBVE} Size \{wSBVE-wSBDS}"
		MESSAGE "     remaining \{dwLVST-wSBVE}"
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end size info

;############################################################# EOF SENDBIN.ASL
		END
